'use strict';

require('../../lib/init');

var assert = require('assert'),
    ModulesTree = require('../../lib/configuration/modules-tree'),
    ReferenceResolver = require('../../lib/manipulation/reference-resolver'),
    Namespacer = require('../../lib/configuration/namespacer'),
    Processor = require('../../lib/configuration/processor'),
    GlobalSectionProcessor = require('../fixtures/configuration/section-processor')
;

var modulesTree = new ModulesTree('main'),
    configurationResolver = require('../fixtures/configuration/configuration-resolver'),
    referenceResolver = new ReferenceResolver(),
    namespacer = new Namespacer(),
    processor = new Processor(configurationResolver, referenceResolver, namespacer, 'test'),
    globalSectionProcessor = new GlobalSectionProcessor('global', configurationResolver, referenceResolver)
;

var expectedConfig = {
        global: {
            value: 5
        },
        'danf': {},
        'main:module1': {
            test: {
                value: 0,
                text: 'text'
            },
            value: 1
        },
        'main:module2': {
            test: 2
        },
        'main:module1:module3': {
            value: 5,
            foo: 'bar'
        },
        main: {
            test: 0
        }
    }
;

modulesTree.build(require('../fixtures/configuration/danf'));
processor.addSectionProcessor(globalSectionProcessor);

describe('Processor', function() {
    describe('"process" method', function() {
        it('should merge and process the configuration of a module and its submodules', function(){
            var config = processor.process(modulesTree);

            assert.deepEqual(config, expectedConfig);
        })
    })
})